import React from 'react';

import { useSelector } from 'react-redux';
import { geoPoint, lat2tile, lon2tile, Point } from '../../lib/geo';

import { MapState } from '../../store/map';
import { getSTile } from './Reticle';
import { tileProviders } from './tile';

interface AvatarForLocationProps {
  location: geoPoint;
  zoom?: number;
  tileProvider?: string;
  size?: number;
  name?: string;
}

const AvatarForLocation: React.FC<AvatarForLocationProps> = (
  props: AvatarForLocationProps
) => {
  const size = props.size ? props.size : 42;
  const zoom = props.zoom ? Math.round(props.zoom) : 16;
  const tileProvider = props.tileProvider ? props.tileProvider : 'osm';
  const location = props.location;
  const name = props.name ? props.name : '???';

  const tilesLocation: Point = {
    x: lon2tile(location.lon, zoom),
    y: lat2tile(location.lat, zoom),
  };

  const locationWithinTile: Point = {
    x: 256 * (tilesLocation.x - Math.floor(tilesLocation.x)),
    y: 256 * (tilesLocation.y - Math.floor(tilesLocation.y)),
  };

  const eastTileNeeded = locationWithinTile.x > 256 - size;
  const westTileNeeded = locationWithinTile.x < size;
  const southTileNeeded = locationWithinTile.y > 256 - size;
  const northTileNeeded = locationWithinTile.y < size;

  const getImage = (stepX: number, stepY: number) => (
    <image
      href={tileProviders[tileProvider].getTileUrl(
        zoom,
        Math.floor(tilesLocation.x) + stepX,
        Math.floor(tilesLocation.y) + stepY
      )}
      height='256'
      width={256}
      x={-locationWithinTile.x + stepX * 256}
      y={-locationWithinTile.y + stepY * 256}
    />
  );

  var images = [getImage(0, 0)];
  if (eastTileNeeded) {
    images.push(getImage(1, 0));
  }
  if (westTileNeeded) {
    images.push(getImage(-1, 0));
  }
  if (southTileNeeded) {
    images.push(getImage(0, 1));
  }
  if (northTileNeeded) {
    images.push(getImage(0, -1));
  }
  if (northTileNeeded && eastTileNeeded) {
    images.push(getImage(1, -1));
  }
  if (northTileNeeded && westTileNeeded) {
    images.push(getImage(-1, -1));
  }
  if (southTileNeeded && eastTileNeeded) {
    images.push(getImage(1, 1));
  }
  if (southTileNeeded && westTileNeeded) {
    images.push(getImage(-1, 1));
  }

  console.log(`${images.length} images: ${images}`);

  return (
    <svg width={size} height={size}>
      <defs>
        <clipPath id='cut-off'>
          <circle cx={size / 2} cy={size / 2} r={size / 2} />
        </clipPath>
      </defs>
      <g clip-path='url(#cut-off)'>
        {[images]}
        <text x='2' y={size / 2 + 5} visibility='hidden'>
          {name}
        </text>
      </g>
    </svg>
  );
};

export default AvatarForLocation;
