import {
  IonModal,
  IonToolbar,
  IonTitle,
  IonButtons,
  IonButton,
  IonContent,
  IonLabel,
  IonList,
  IonListHeader,
  IonItem,
  IonSpinner,
} from '@ionic/react';
import React, { Fragment, useRef, useState } from 'react';
import {
  MainContainer,
  Sidebar,
  ConversationList,
  Conversation,
  ConversationHeader,
  ChatContainer,
  MessageList,
  Message,
  Avatar,
} from '@chatscope/chat-ui-kit-react';
import '@chatscope/chat-ui-kit-styles/dist/default/styles.min.css';

import { useSelector } from 'react-redux';
import i18n from '../../i18n';
import { geoPoint } from '../../lib/geo';
import { MapState } from '../../store/map';
import AvatarForLocation from './AvatarForLocation';

const LocationInfo: React.FC<{}> = () => {
  const scope = useSelector((state: { map: MapState }) => state.map.scope);

  const modal = useRef<HTMLIonModalElement>(null);

  const dismiss = () => {
    modal.current?.dismiss();
  };

  const [elevation, setElevation] = useState(undefined);
  const [address, setAddress] = useState<{ display_name: any } | undefined>(
    undefined
  );
  const [notes, setNotes] = useState<{ features: any } | undefined>(undefined);

  const findElevation = async () => {
    setElevation(undefined);
    const response = await fetch(
      //     `https://api.opentopodata.org/v1/mapzen?locations=${scope.center.lat},${scope.center.lon}`,
      `https://api.open-meteo.com/v1/elevation?latitude=${scope.center.lat}&longitude=${scope.center.lon}`,
      {}
    );
    const data = await response.json();
    console.log(`elevation: ${JSON.stringify(data)}`);
    setElevation(data.elevation[0]);
  };

  const findAddress = async () => {
    setAddress(undefined);
    const response = await fetch(
      `https://nominatim.openstreetmap.org/reverse?lat=${
        scope.center.lat
      }&lon=${
        scope.center.lon
      }&format=jsonv2&addressdetails=1&extratags=1&namedetails=1&accept-language=${i18n.getLanguage()}`,
      {}
    );
    const data = await response.json();
    console.log(`address: ${JSON.stringify(data)}`);
    setAddress(data);
  };

  const findNotes = async () => {
    setNotes(undefined);
    setNoteIndex(0);
    const metresPerDegree =
      111111 * Math.cos((scope.center.lat * Math.PI) / 180);
    const deltaDegrees = 2000 / metresPerDegree;
    const response = await fetch(
      `https://api.openstreetmap.org/api/0.6/notes.json?bbox=${
        scope.center.lon - deltaDegrees
      },${scope.center.lat - deltaDegrees},${scope.center.lon + deltaDegrees},${
        scope.center.lat + deltaDegrees
      }`,
      {}
    );
    const data = await response.json();
    console.log(`notes: ${JSON.stringify(data)}`);
    setNotes(data);
  };

  const ionModalWillPresentHandler = async () => {
    findElevation();
    findAddress();
    findNotes();
  };

  const roughDistance = (a: geoPoint, b: geoPoint): number => {
    const pseudoDistanceInDegrees = Math.sqrt(
      (a.lat - b.lat) ** 2 + (a.lon - b.lon) ** 2
    );
    const metresPerDegree =
      111111 * Math.cos((scope.center.lat * Math.PI) / 180);
    return pseudoDistanceInDegrees * metresPerDegree;
  };

  const localDate = (isoDate: string) => {
    const date = new Date(isoDate);
    const format = new Intl.DateTimeFormat(i18n.getLanguage(), {
      dateStyle: 'full',
      timeStyle: 'long',
    });
    return format.format(date);
  };

  const [noteIndex, setNoteIndex] = useState(0);
  const conversationClickHandlerFactory = (index: number) => (event: any) => {
    setNoteIndex(index);
  };

  const AvatarForFeature: React.FC<{ feature: any; as: any }> = (props: {
    feature: any;
    as: any;
  }) => {
    const distance = roughDistance(scope.center, {
      lon: props.feature.geometry.coordinates[0],
      lat: props.feature.geometry.coordinates[1],
    });
    const distDark = (255 * distance) / 3000;
    const distLight = 1 - distDark;
    return (
      <Avatar>
        <AvatarForLocation
          location={{
            lon: props.feature.geometry.coordinates[0],
            lat: props.feature.geometry.coordinates[1],
          }}
          name={Math.round(distance).toString()}
        />
      </Avatar>
    );
  };

  return (
    <IonModal
      trigger='information-request'
      ref={modal}
      onIonModalWillPresent={ionModalWillPresentHandler}
    >
      <IonToolbar>
        <IonTitle>{i18n.locationInfo!.title}</IonTitle>
        <IonButtons slot='end'>
          <IonButton onClick={() => dismiss()}>{i18n.close}</IonButton>
        </IonButtons>
      </IonToolbar>
      <IonContent>
        <IonList lines='full' class='ion-no-margin'>
          <IonListHeader lines='full'>
            <IonLabel>{i18n.locationInfo!.location}</IonLabel>
          </IonListHeader>
          <IonItem>
            <IonLabel>{i18n.locationInfo!.lat}</IonLabel>
            {scope.center.lat} N
          </IonItem>
          <IonItem>
            <IonLabel>{i18n.locationInfo!.lon}</IonLabel>
            {scope.center.lon} E
          </IonItem>
          <IonItem>
            <IonLabel>{i18n.locationInfo!.elevation}</IonLabel>
            {elevation === undefined && <IonSpinner name='lines' />}
            {elevation !== undefined && <>{elevation} m</>}
          </IonItem>
        </IonList>
        <IonList lines='full' class='ion-no-margin'>
          <IonListHeader lines='full'>
            <IonLabel>{i18n.locationInfo!.address}</IonLabel>
          </IonListHeader>
          <IonItem>
            <IonLabel>{i18n.locationInfo!.display_name}</IonLabel>
            {address === undefined && <IonSpinner name='lines' />}
            {address !== undefined &&
              address.display_name
                .split(',')
                .map((value: string, index: number) => (
                  <Fragment key={index}>
                    {value}
                    <br />
                  </Fragment>
                ))}
          </IonItem>
        </IonList>
        <IonList lines='full' class='ion-no-margin'>
          <IonListHeader lines='full'>
            <IonLabel>
              {i18n.locationInfo!.notes}
              {notes === undefined && <IonSpinner name='lines' />}
            </IonLabel>
          </IonListHeader>
          {notes !== undefined && notes.features.length > 0 && (
            <div style={{ position: 'relative' }}>
              <MainContainer responsive>
                <Sidebar position='left' id='chat-sidebar'>
                  <ConversationList>
                    {notes.features!.map((feature: any, index: number) => (
                      <Conversation
                        onClick={conversationClickHandlerFactory(index)}
                        key={feature.properties.id}
                        name={i18n.locationInfo!.at!(
                          roughDistance(scope.center, {
                            lon: feature.geometry.coordinates[0],
                            lat: feature.geometry.coordinates[1],
                          })
                        )}
                        info={feature.properties.status}
                      >
                        <AvatarForFeature feature={feature} as={Avatar} />
                      </Conversation>
                    ))}
                  </ConversationList>
                </Sidebar>
                <ChatContainer>
                  <ConversationHeader>
                    <ConversationHeader.Content>
                      <span
                        style={{
                          alignSelf: 'flex-center',
                        }}
                      >
                        {i18n.locationInfo!.at!(
                          roughDistance(scope.center, {
                            lon: notes.features[noteIndex].geometry
                              .coordinates[0],
                            lat: notes.features[noteIndex].geometry
                              .coordinates[1],
                          })
                        )}
                      </span>
                    </ConversationHeader.Content>
                    <AvatarForFeature
                      feature={notes.features[noteIndex]}
                      as={Avatar}
                    />
                  </ConversationHeader>
                  <MessageList>
                    {noteIndex < notes.features.length &&
                      notes.features[noteIndex].properties.comments.map(
                        (comment: any, index: number) => (
                          <Message
                            key={`${notes.features[noteIndex].properties.id}/${index}`}
                            model={{
                              direction: 'incoming',
                              position: 'single',
                              sender: comment.user,
                              sentTime: comment.date,
                              payload: comment.html,
                            }}
                          >
                            <Message.Header
                              sender={comment.user}
                              sentTime={comment.date}
                            />
                            <Message.Footer sentTime={comment.action} />
                            <Avatar>
                              {comment.user ? comment.user : '??'}
                            </Avatar>
                          </Message>
                        )
                      )}
                  </MessageList>
                </ChatContainer>
              </MainContainer>
            </div>
          )}
        </IonList>
      </IonContent>
    </IonModal>
  );
};

export default LocationInfo;
